function [env] = create_orthogonal_patterns_for_recognition(n_disrupt_patterns, pres_indices, rec_indices, ...
    not_presented_indices,ffr_indices,recog_indicies)
%CREATE_ORTHOGONAL_PATTERNS   Create a set of orthogonal patterns.
%
%  Creates a set of patterns that can be drawn from to make lists.
%  This function creates LOCALIST, BINARY, ORTHOGONAL patterns
%
%  [env] = create_orthogonal_environment(n_patterns, pres_indices, ...
%	 				not_presented_indices);
%
%  INPUTS:
%
%            n_patterns - How many distinct patterns are required for
%                         each sub-region.  A vector of length
%                         subregions.
%
%          pres_indices - A cell array containing the indices of the
%                         items that will be presented to the
%                         network. If there is an associated
%                         semantic matrix, these indices reference
%                         that matrix, otherwise, each unique item
%                         simply has a unique integer associated
%                         with it. Dimensions are:
%                         {trial}(subregion, serial position)
%
% not_presented_indices - A cell array.  Each cell contains the
%                         indices of the items that are recallable
%                         by the network but are not ever presented
%                         to the network.
%
%  OUTPUTS:
%          env.patterns - A cell array.  Each cell contains a
%                         matrix with dimensions (num_patterns,
%                         num_dimensions_in_subregion).
%
%  env.pool_to_item_map - A cell array.  Each cell contains a
%                         matrix with dimensions (num_patterns,
%                         2).  The first column contains the pool
%                         index of each presented item.
%
%       env.pat_indices - A cell array, the same size as pres_indices,
%                         that contains the indices for each
%                         presented item referenced to the patterns
%                         matrix. Dimensions are
%                         {trial}(subregion, serial position)
%                         env.patterns(env.pat_indices{1}(1,1) would
%                         return the first presented pattern.

% env.pat_indices indexes the patterns
env.pat_indices = cell(size(pres_indices));
env.rec_indices = cell(size(rec_indices));

pres_indices_matrix = [pres_indices{:}];
rec_indices_matrix = [rec_indices{:}];


subregion_pres_indices = pres_indices_matrix(1,:);
subregion_rec_indices = rec_indices_matrix(1,:);

all_unique = unique([subregion_pres_indices(:);subregion_rec_indices(:); ...
    not_presented_indices{1}(:);...
    ffr_indices{1}(:);recog_indicies{1}(:)]);
all_unique = all_unique(all_unique~=0); %get rid of zeros, which are just used as padding
n_unique = length(all_unique);

% add in values for end of list, and end of phase distractors
n_patterns = n_unique + n_disrupt_patterns;

% record the first distraction index
env.first_distraction_index = n_unique + 1;

% environment patterns
env.patterns{1} = eye(n_patterns);% setting to the number of unique items. Originally specified as a param: eye(n_patterns(subregion));
env.pool_to_item_map{1}(:,1) = all_unique;
env.pool_to_item_map{1}(:,2) = 1:n_unique;

for trial = 1:size(pres_indices,2)
    trial_pres_indices = pres_indices{trial}(1,:);
    
    for item = 1:length(trial_pres_indices)
        env.pat_indices{trial}(1,item) = ...
            env.pool_to_item_map{1}(find(trial_pres_indices(item)==...
            env.pool_to_item_map{1}(:,1)),2);
        
    end
    
    trial_rec_indices = rec_indices{trial}(1,:);
    
    % get rid of the zeros which are just padding
    trial_rec_indices = trial_rec_indices(trial_rec_indices ~= 0);
    
    for item = 1:length(trial_rec_indices)
        env.rec_indices{trial}(1,item) = ...
            env.pool_to_item_map{1}(find(trial_rec_indices(item)==...
            env.pool_to_item_map{1}(:,1)),2);
        
    end
    
end



%
% for subregion = 1:length(n_patterns)
%     subregion_pres_indices = pres_indices_matrix(subregion,:);
%     subregion_rec_indices = rec_indices_matrix(subregion,:);
%
%     all_unique = unique([subregion_pres_indices(:);subregion_rec_indices(:); ...
%         not_presented_indices{subregion}(:);...
%         ffr_indices{subregion}(:);recog_indicies{subregion}(:)]);
%     all_unique = all_unique(all_unique~=0); %get rid of zeros, which are just used as padding
%     n_unique = length(all_unique);
%
%     % environment patterns
%     env.patterns{subregion} = eye(n_unique);% setting to the number of unique items. Originally specified as a param: eye(n_patterns(subregion));
%     env.pool_to_item_map{subregion}(:,1) = all_unique;
%     env.pool_to_item_map{subregion}(:,2) = 1:n_unique;
%
%     for trial = 1:size(pres_indices,2)
%         trial_pres_indices = pres_indices{trial}(subregion,:);
%
%         for item = 1:length(trial_pres_indices)
%             env.pat_indices{trial}(subregion,item) = ...
%                 env.pool_to_item_map{subregion}(find(trial_pres_indices(item)==...
%                 env.pool_to_item_map{subregion}(:,1)),2);
%
%         end
%
%         trial_rec_indices = rec_indices{trial}(subregion,:);
%
%         % get rid of the zeros which are just padding
%         trial_rec_indices = trial_rec_indices(trial_rec_indices ~= 0);
%
%         for item = 1:length(trial_rec_indices)
%             env.rec_indices{trial}(subregion,item) = ...
%                 env.pool_to_item_map{subregion}(find(trial_rec_indices(item)==...
%                 env.pool_to_item_map{subregion}(:,1)),2);
%
%         end
%
%     end
end
