function recog_struct = run_LTP_cmr2_recognition(param)
% function data = run_K02_iy(param,datapath)
%
% Simulate the IFR condition of Kahana et al. 2002. This function also is
% generalizable enough to run the control lists of taskFR when task
% information is not included.
%
% INPUTS param - a parameters structure specifying the configuration of the
% model.
%
% datapath - path to the matlab structure containing the behavioral data
% from the experiment. the data structure must have the following fields:
%
% pres_indices: each row represents one trial of free recall, with the
% items in the row provided in the order in which they should be presented
% to the model. items are indexed globally with respect to all items in the
% word pool. not_presented indices: for each subject, one list of items not
% presented to the model but available for recall. these lists should be
% given in the same order as the order of subjects (e.g. the first subject
% has the first list), which may differ from the order of subjects ordered
% numerically. subject: for each presented trial row, a number indicating
% the subject to which that trial was presented. session: for each
% presented trial row, a number indicating the session.
%
% OUTPUTS data -the data structure will have the same fields as the data
% structure loaded from datapath. added on will be the following fields:
%
% taken directly from the original fields: net.pres_indices net.subject
%
% generated from the model simulations: net.recalls: for each trial, the
% items recalled by the model, indexed according to serial position. items
% recalled that are not from the most recently presented list are indexed
% as -1. each column is an output position. net.rec_itemnos: for each
% trial, the items recalled indexed according to their global number with
% respect to the word pool. net.times: for each recalled item, the
% cumulative time that has passed before the item is recalled.
%
% calculated from other fields: .net.list_length: the number of items
% presented on each list. here, this number is constant across subjects.
% .net.intrusions: for each trial (row) and output position (column),
% indicates the type of intrusion that was made:
%
%       0: no intrusion
%
%       -1: extra-list intrusion (XLI)
%
%       positive integer: prior-list intrusion (PLI), indicating the number
%       of lists back from which the PLI was recalled

param.lrate_fc_enc = param.gamma_fc;

param.eye_fc = 1 - param.gamma_fc;

param.lrate_cf_enc = param.gamma_cf;

param.eye_cf = 1 - param.gamma_cf;

load(param.data_path);
load(param.ffr_data_path);
load(param.recog_data_path);

% make sure same subjects in each disp('this section should go in the
% initial file creations script - nuts to do this 1000''s of times')
% pause(2) common_ss =
% intersect(intersect([data.subject],[ffr_struct.subject]),[recog_struct.subject])
% data = trial_subset(ismember(data.subject,common_ss),data)

% extract LSA information.
load(param.sem_path);
param.sem_mat{1} = LSA;
clear LSA

subjlist = param.subjlist;
%subjlist = unique(data.subject);

% set variables.
num_trials = length(data.subject);
list_length = size(data.pres_indices,2);
param.max_outputs = list_length;
trial_count = 1;
nsubj = param.nsubj;

param.end_schedule = ones(30,1) * param.B_end_list;

% loop through per subject.
for subject = 1:nsubj
    tic
    %subject./nsubj
    % get session info for this subject.
    this_subj_session = data.session(data.subject==subjlist(subject));
    
    % loop through sessions per subject.
    session_list = (unique(this_subj_session))';
    for session = 1:size(session_list,2)
        
        % extract out the relevant rows.
        this_session_rows = and(data.subject==subjlist(subject),data.session==session_list(session));
        
        % set the items presented to the subject for this session
        % (pres_indices)
        param.pres_indices = {};
        temp_pres_indices = data.pres_indices(this_session_rows,:);
        
        % set the items recalled by this subject for this sessions
        param.rec_indices = {};
        temp_rec_indices = data.rec_indices(this_session_rows,:);
        
        
        
        
        
        % each subject additionally has one list of items not presented to
        % the model but available for recall (not_presented_indices)
        param.not_presented_indices{1} = data.not_pres_indices{subject,session};
        
        
        % add the indicies for FFR items and recog items to paams isolate
        % ffr items from this session
        ffr_struct_i = ...
            ffr_struct([ffr_struct.subject]==subjlist(subject) & [ffr_struct.session]==session_list(session));
        ffr_indicies = [ffr_struct_i.itemno];
        ffr_indicies(isnan([ffr_struct_i.studytrial])) = -1;% set prior session intrusions to -1
        
        
        % the indicies to this sessions entries in recog_struct. We will
        % need these later on.
        recog_lines_i = find([recog_struct.subject]==subjlist(subject) & ...
            [recog_struct.session]==session_list(session));
        
        % grab the item numbers of the recog probes for this session
        recog_struct_i = recog_struct(recog_lines_i);
        recog_indicies = [recog_struct_i.itemno];
        
        
        
        
        % replace intrusions in rec_indicies with a word from not pres pool
        intru = temp_rec_indices == -1;
        n_not_pres = length(param.not_presented_indices{1}(:));
        this_one = randsample(1:n_not_pres,sum(intru(:)));
        temp_rec_indices(intru) = param.not_presented_indices{1}(this_one);
        param.not_presented_indices{1} =  param.not_presented_indices{1}(~ismember(1:n_not_pres,this_one));
        
        % replace intrusions in ffr_indicies with a word from not pres pool
        intru = ffr_indicies == -1;
        n_not_pres = length(param.not_presented_indices{1}(:));
        this_one = randsample(1:n_not_pres,sum(intru(:)));
        ffr_indicies(intru) = param.not_presented_indices{1}(this_one);
        param.not_presented_indices{1} =  param.not_presented_indices{1}(~ismember(1:n_not_pres,this_one));
        
        
        
        % reformat for the model.
        param.ffr_indices{1} = ffr_indicies;
        param.recog_indicies{1} = recog_indicies;
        for j = 1:size(temp_pres_indices,1)
            param.pres_indices{j} = temp_pres_indices(j,:);
            param.rec_indices{j} = temp_rec_indices(j,:);
        end
        
        
        %% recall
        
        % We are not actually simulating recall, we are just setting the
        % network to the state it would be in after the subject's actual
        % sequence of recalls. Basically, we present the network with both
        % the study items, AND the items the participant recalled, allowing
        % associations to form and context to drift as per model params,
        % but not running the accumulators to decide which items are
        % recalled.
        [net,env] = run_fr_for_recognition(param);
        %trial(trial_count) = simulate_fr_abridged(param);
        trial_count = trial_count + 1;
        
       
        %% ffr
        
        % do the same for the final free recall period, if there was one
        % for this session.
        
        
        
        if ~isempty(ffr_struct_i)
            
            % context drift between phases
            env.present_index(logical(param.end_disrupt_regions)) = ...
                env.present_distraction_index(logical(param.end_disrupt_regions));
            net = present_distraction(net, env, ...
                param.end_disrupt_regions, ...
                param.b_ffr, param);
            env.present_distraction_index = env.present_distraction_index + 1;
            
            
            for item = 1:length(ffr_indicies)
                env.rec_indices{1}(1,item) = ...
                    env.pool_to_item_map{1}(find(ffr_indicies(item)==...
                    env.pool_to_item_map{1}(:,1)),2);
                
            end
            
            
            
            
            env.recalled_items = env.rec_indices{1};
            
            [net, env] = param.recall_task_fn(net, env, param);
            
        end
        

        
        %% recognition
        
        % context drift between phases
            env.present_index(logical(param.end_disrupt_regions)) = ...
                env.present_distraction_index(logical(param.end_disrupt_regions));
            net = present_distraction(net, env, ...
                param.end_disrupt_regions, ...
                param.b_recog, param);
            
        
        
        % make a blank feature layer here to avoid having to make one on
        % each loop
        blank_f = zeros(size(net.f));
        
        % set proper beta value for dirft upon recollection
        net.c_sub{1}.B = param.b_r;
        
        % loop over probes
        n_probes = length(recog_indicies);
        for probe_i = 1:n_probes
            
            % get pool_to_item indicies for the probe
            env.recog_indices{1}(1,probe_i) = ...
                env.pool_to_item_map{1}(find(recog_indicies(probe_i)==...
                env.pool_to_item_map{1}(:,1)),2);
            
            probe_idx = ...
                env.pool_to_item_map{1}(find(recog_indicies(probe_i)==...
                env.pool_to_item_map{1}(:,1)),2);
            
            % activate the probe on the item layer. You could do this using
            % the: net.f(net.f_sub{1}.idx) = env.patterns{1}(probe_idx,:);
            % method the reactivate_item_abridged uses, but it is over 2X
            % faster to do it this way
            net.f = blank_f;
            net.f(probe_idx) = 1;
            
            % allow the probe to retreive its context
            net.c_in = net.w_fc * net.f;
            
            % compare the retreived context to c_i
            cdotc_in = sum(net.c_in .* net.c);
            
            % mark as a yes response if it passes threshold, as a no
            % response otherwise
            if cdotc_in > param.recog_thresh
                recog_struct(probe_idx).recog_resp = true;
                net = param.custom_context_fn{1}(net, 1, param);
            else
                recog_struct(probe_idx).recog_resp = false;
            end
            
            
            
        end
        
        
        
        
        
        
    end
    toc
end
%
%     % grab the useful fields from the original data structure
%     data.net.subject = data.subject(1:num_trials); data.net.pres_indices
%     = data.pres_indices(1:num_trials,:); data.net.session =
%     data.session(1:num_trials,:);
%
%     % grab the fields created by simulate_fr, and string together f =
%     fieldnames(trial(1)); for i = 1:length(f)
%         data.net.(f{i}) = [];
%     end
%
%     % find max number of columns in case the different trials differ
%     maxcols = 0; for i = 1:length(trial)
%         ncols = size(trial(i).recalls,2); maxcols = max(ncols, maxcols);
%     end
%
%     % stitch the fields together to make a data structure for trial_num =
%     1:length(trial)
%         for field_name = 1:length(f)
%             this_field = trial(trial_num).(f{field_name}); ncols =
%             size(this_field,2); if ncols < maxcols
%                 this_field(:,ncols+1:maxcols) = 0;
%             end data.net.(f{field_name}) =
%             single([data.net.(f{field_name}); this_field]);
%         end
%     end
%
%     % construct the intrusions field %We want to distinguish between XLIs
%     that come from lists the subject has %not seen yet, but eventually
%     will (i.e., 'Future List Intrusions') and %true XLIs that are words
%     that have not been, and will not ever be, %presetned to the subject.
%     FLI_make_intrusions does this. Use plain old %make_intrusions if you
%     want to count FLIs as XLIs. data.net.intrusions =
%     FLI_make_intrusions(data.net.rec_itemnos,data.net.pres_indices,data.net.subject);
%
%     % set other field data.net.list_length = list_length;